#import "CaptureflutterPlugin.h"
#import "pigeon.h"
#import "SKTCapture.h"
#import "CaptureFlutterHandle.h"
@interface CaptureFlutterImplementation: NSObject<CaptureFlutter, SKTCaptureDelegate, FlutterStreamHandler> {
    CaptureFlutterHandle* _handles;
    FlutterEventSink _flutterEvent;
}
// - (void)openClientAppInfo:(nullable AppInfo *)appInfo completion:(void(^)(CaptureHandle *_Nullable, FlutterError *_Nullable))completion;
// - (void)openDeviceGuid:(nullable NSString *)guid completion:(void(^)(CaptureHandle *_Nullable, FlutterError *_Nullable))completion;
// - (void)closeHandle:(nullable CaptureHandle *)handle completion:(void(^)(CaptureResult *_Nullable, FlutterError *_Nullable))completion;
// - (void)getPropertyProperty:(nullable Property *)property completion:(void(^)(Property *_Nullable, FlutterError *_Nullable))completion;
// - (void)setPropertyProperty:(nullable Property *)property completion:(void(^)(Property *_Nullable, FlutterError *_Nullable))completion;
@end

@implementation CaptureFlutterImplementation
- (void)openClientAppInfo:(nullable AppInfo *)appInfo completion:(void(^)(CaptureHandle *_Nullable, FlutterError *_Nullable))completion{
  SKTAppInfo* sktAppInfo = [SKTAppInfo new];
  sktAppInfo.DeveloperID = appInfo.developerId;
  sktAppInfo.AppID = appInfo.appId;
  sktAppInfo.AppKey = appInfo.appKey;
  SKTCapture* capture = [[SKTCapture alloc] initWithDelegate: self];

    if(self->_handles == nil){
        self->_handles = [CaptureFlutterHandle new];
    }
    NSLog(@"Just before calling Capture openWithAppInfo");
    [capture openWithAppInfo:sktAppInfo completionHandler:^(SKTResult result){
        CaptureHandle* handle = nil;
        FlutterError* res = nil;
        if(result < SKTCaptureE_NOERROR){
            res = [FlutterError errorWithCode:[NSString stringWithFormat:@"%ld", (long)result] message:@"Unable to open Capture" details:nil];
        }
        else{
          handle = [CaptureHandle new];
          handle.value = [self->_handles addNewObject:capture];
        }
        completion(handle, res);
        NSLog(@"Done opening Capture");
    }];
}

- (void)openDeviceGuid:(nullable NSString *)guid completion:(void(^)(CaptureHandle *_Nullable, FlutterError *_Nullable))completion{

}
- (void)closeHandle:(nullable CaptureHandle *)handle completion:(void(^)(CaptureResult *_Nullable, FlutterError *_Nullable))completion{
    if(_handles != nil){
        SKTCapture* capture = (SKTCapture*)[_handles removeObjectFromHandle:[handle value]];
        [capture closeWithCompletionHandler:^(SKTResult result) {
            if(result>= SKTCaptureE_NOERROR){
                CaptureResult* res = [CaptureResult new];
                res.value = [NSNumber numberWithLong: result];
                completion(res, nil);
            }
            else {
                FlutterError* err = [FlutterError errorWithCode:[NSString stringWithFormat:@"%ld",(long)result] message:@"Unable to close with the provided handle" details:nil];
                completion(nil, err);
            }
        }];
    }
}
- (void)getPropertyProperty:(nullable Property *)property completion:(void(^)(Property *_Nullable, FlutterError *_Nullable))completion{

}
- (void)setPropertyProperty:(nullable Property *)property completion:(void(^)(Property *_Nullable, FlutterError *_Nullable))completion{

}
- (void)didReceiveEvent:(SKTCaptureEvent * _Nonnull)event forCapture:(SKTCapture * _Nonnull)capture withResult:(SKTResult)result {
    NSLog(@"Capture didReceiveEvent");
    if(_flutterEvent != nil){
        (_flutterEvent)(@"DidReceiveEvent from Capture");
    }
}

- (FlutterError * _Nullable)onCancelWithArguments:(id _Nullable)arguments {
    FlutterError* error = nil;
    return error;
}

- (FlutterError * _Nullable)onListenWithArguments:(id _Nullable)arguments eventSink:(nonnull FlutterEventSink)events {
    FlutterError* error = nil;
    
    _flutterEvent = events;
    
    return error;
}


@end


@implementation CaptureflutterPlugin
+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    
    // demo to retrieve the iOS version
  FlutterMethodChannel* channel = [FlutterMethodChannel
      methodChannelWithName:@"captureflutter"
            binaryMessenger:[registrar messenger]];
  CaptureflutterPlugin* instance = [[CaptureflutterPlugin alloc] init];
  [registrar addMethodCallDelegate:instance channel:channel];
  
  // Capture API done with Pigeon
  CaptureFlutterImplementation* capture = [CaptureFlutterImplementation new];
  CaptureFlutterSetup([registrar messenger], capture);

  // Event Channel for Capture Event notifications
  FlutterEventChannel* eventChannel = [FlutterEventChannel 
                                    eventChannelWithName:@"captureevent" 
                                        binaryMessenger:[registrar messenger]];
    [eventChannel setStreamHandler:capture];
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
  if ([@"getPlatformVersion" isEqualToString:call.method]) {
    result([@"iOS " stringByAppendingString:[[UIDevice currentDevice] systemVersion]]);
  } else {
    result(FlutterMethodNotImplemented);
  }
}

@end
